import { decode } from 'vlq';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __spreadArrays() {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
}

/**
 * Accumulate map by vlq code
 * @param origin Origin map
 * @param vlq    Vlq code
 */
var accMap = function (origin, vlq) {
    var source = decode(vlq);
    origin[0] = (origin[0] || 0) + (source[0] || 0);
    origin[1] = (origin[1] || 0) + (source[1] || 0);
    origin[2] = (origin[2] || 0) + (source[2] || 0);
    origin[3] = (origin[3] || 0) + (source[3] || 0);
    origin[4] = (origin[4] || 0) + (source[4] || 0);
    return origin;
};
/**
 * View the original code from the sourcemap based on the error lineno and colno.
 * @param {string} sourcemap
 */
var sourcemapView = function (sourcemap) {
    var mappings = (sourcemap.mappings || '').split(';').map(function (line) { return line.split(','); });
    // Cache generated line map
    var prevLineMaps = [];
    return function (lineNo, colNo) {
        var line = lineNo - 1;
        var col = colNo - 1;
        // Wrong line.
        if (!mappings[line])
            return null;
        // Init the map, move to previous line map if it exists.
        var map = prevLineMaps[line - 1] || [];
        // Recaculate when previous line map is not found.
        if (line > 0 && line > prevLineMaps.length) {
            map = prevLineMaps[prevLineMaps.length - 1] || [];
            mappings.some(function (lineItem, index) {
                if (index < prevLineMaps.length)
                    return false;
                map[0] = 0; // reset column of next line
                lineItem.forEach(function (item) { return accMap(map, item); });
                prevLineMaps.push(__spreadArrays(map));
                return index + 1 >= line;
            });
        }
        map[0] = 0; // reset column of next line
        // Pinpoint the map in current line.
        mappings[line].some(function (item) {
            accMap(map, item);
            return map[0] >= col;
        });
        // Wrong col.
        if (map[0] !== col && map[0] !== colNo)
            return null;
        return {
            file: Number.isInteger(map[1]) && sourcemap.sources ? sourcemap.sources[map[1]] : null,
            lineNo: Number.isInteger(map[2]) ? map[2] + 1 : 0,
            colNo: Number.isInteger(map[3]) ? map[3] : 0,
            name: Number.isInteger(map[4]) && sourcemap.names ? sourcemap.names[map[4]] : null,
            content: Number.isInteger(map[1]) && sourcemap.sourcesContent ? sourcemap.sourcesContent[map[1]] : null,
        };
    };
};

export default sourcemapView;
